program AppStart;

{$mode objfpc}{$H+}

uses
  Forms,
  Windows,
  {$IFDEF UNIX}
  cthreads,
  {$ENDIF}
  Classes, SysUtils, CustApp
  { you can add units after this },
  IniFiles, Interfaces;

type

  { TMyApplication }

  TMyApplication = class(TCustomApplication)
  protected
    procedure DoRun; override;
  public
    ApplicationName: String;
    UseGui:          Boolean;
    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;
    procedure WriteHelp; virtual;
    procedure ReadSettings;
    procedure WriteSettings;
    procedure CreateSettingForm;
  end;

const
  ExeFileName  = 'MyApp.exe';
  IniFile      = 'AppStart.ini';

{ TMyApplication }

procedure TMyApplication.DoRun;
var
  ErrorMsg: String;
begin
  // quick check parameters
  ErrorMsg:=CheckOptions('h', 'help');
  if ErrorMsg<>'' then begin
    ShowException(Exception.Create(ErrorMsg));
    Terminate;
    Exit;
  end;

  // parse parameters
  if HasOption('h', 'help') then begin
    WriteHelp;
    Terminate;
    Exit;
  end;

  { add your program here }
  if not FileExists(IniFile) then
  begin
    ApplicationName := ExeFileName;
    UseGui          := True;
    WriteSettings;
  end;
  ReadSettings;

  if UseGui then
     CreateSettingForm;

  // stop program loop
  Terminate;
end;

constructor TMyApplication.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  StopOnException:=True;
end;

destructor TMyApplication.Destroy;
begin
  inherited Destroy;
end;

procedure TMyApplication.WriteHelp;
begin
  { add your help code here }
  writeln('Usage: ', ExeName, ' -h');
end;

procedure TMyApplication.ReadSettings;
var
  IniFileSettings: TIniFile;

begin
  IniFileSettings := TIniFile.Create(IniFile);
  ApplicationName := IniFileSettings.ReadString('Main', 'ApplicationName', ExeFileName);    // (Section, Key, Default)
  UseGui          := IniFileSettings.ReadBool(  'Main', 'UseGui',          true);
  IniFileSettings.Free;
end;

procedure TMyApplication.WriteSettings;
var
  IniFileSettings: TIniFile;

begin
  IniFileSettings := TIniFile.Create(IniFile);
  IniFileSettings.WriteString('Main', 'ApplicationName', ApplicationName);
  IniFileSettings.WriteBool(  'Main', 'UseGui',          UseGui);
  IniFileSettings.Free;
end;


procedure TMyApplication.CreateSettingForm;
var
  MyForm: TForm;

begin
  MyForm := TForm.Create(nil);
  MyForm.ShowModal;
  FreeAndNil(MyForm);
end;


var
  Application: TMyApplication;
begin
  Application := TMyApplication.Create(nil);
  Application.Run;
  Application.Free;
end.
